@@ -1,7 +1,8 @@
 /*
- * $Id: header_f.h,v 1.3 2004/06/05 17:39:14 calrissian Exp $
+ * $Id: header_f.h,v 1.5 2005/04/10 20:57:28 calrissian Exp $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
+ * Copyright (C) 2004-2005 Nils Ohlmeier
  *
  * This file belongs to sipsak, a free sip testing tool.
  *
@@ -29,6 +30,10 @@ void set_maxforw(char *mes);
 
 void uri_replace(char *mes, char *uri);
 
+void set_cl(char* mes, int contentlen);
+
+void build_ack(char *invite, char *reply, char *ack);
+
 void warning_extract(char *message);
 
 int cseq(char *message);
