@@ -1,7 +1,8 @@
 /*
- * $Id: helper.c,v 1.14 2004/10/08 17:26:00 calrissian Exp $
+ * $Id: helper.c,v 1.23 2005/04/10 20:57:28 calrissian Exp $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
+ * Copyright (C) 2004-2005 Nils Ohlmeier
  *
  * This file belongs to sipsak, a free sip testing tool.
  *
@@ -15,18 +16,31 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  */
+#include "sipsak.h"
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <netdb.h>
-#include <unistd.h>
-#include <sys/utsname.h>
-#include <string.h>
-#include <ctype.h>
-#include <arpa/inet.h>
+#ifdef HAVE_NETDB_H
+# include <netdb.h>
+#endif
+#ifdef HAVE_UNISTD_H
+# ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+# endif
+# include <unistd.h>
+#endif
+#ifdef HAVE_SYS_UTSNAME_H
+# include <sys/utsname.h>
+#endif
+#ifdef HAVE_CTYPE_H
+# include <ctype.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
 
 #include "helper.h"
-#include "sipsak.h"
 #include "exit_code.h"
 
 /* take either a dot.decimal string of ip address or a 
@@ -53,10 +67,10 @@ long getaddress(char *host)
 	/*try understanding if this is a valid ip address
 	we are skipping the values of the octets specified here.
 	for instance, this code will allow 952.0.320.567 through*/
-	while (*p)
+	while (*p != '\0')
 	{
 		for (i = 0; i < 3; i++, p++)
-			if (!isdigit((int)*p))
+			if (isdigit((int)*p) == 0)
 				break;
 		if (*p != '.')
 			break;
@@ -98,7 +112,6 @@ void get_fqdn(){
 	size_t namelen=100;
 	struct hostent* he;
 	struct utsname un;
-	char *fqdnp;
 
 	memset(&hname, 0, sizeof(hname));
 	memset(&dname, 0, sizeof(dname));
@@ -136,9 +149,9 @@ void get_fqdn(){
 
 	he=gethostbyname(hname);
 	if (he) {
-		if (numeric) {
-			sprintf(hlp, "%s", inet_ntoa(*(struct in_addr *) he->h_addr_list[0]));
-			fqdnp = strcat(fqdn, hlp);
+		if (numeric == 1) {
+			snprintf(hlp, 15, "%s", inet_ntoa(*(struct in_addr *) he->h_addr_list[0]));
+			strcpy(fqdn, hlp);
 		}
 		else {
 			if ((strchr(he->h_name, '.'))!=NULL && (strchr(hname, '.'))==NULL) {
@@ -173,7 +186,7 @@ void get_fqdn(){
 void replace_string(char *mess, char *search, char *replacement){
 	char *backup, *insert;
 
-	insert=strstr(mess, search);
+	insert=STRSTR(mess, search);
 	if (insert==NULL){
 		if (verbose > 2)
 			printf("warning: could not find this '%s' replacement string in "
@@ -182,11 +195,15 @@ void replace_string(char *mess, char *search, char *replacement){
 	else {
 		while (insert){
 			backup=malloc(strlen(insert)+1);
+			if (!backup) {
+				printf("failed to allocate memory\n");
+				exit_code(255);
+			}
 			strcpy(backup, insert+strlen(search));
 			strcpy(insert, replacement);
 			strcpy(insert+strlen(replacement), backup);
 			free(backup);
-			insert=strstr(mess, search);
+			insert=STRSTR(mess, search);
 		}
 	}
 }
@@ -197,8 +214,12 @@ void insert_cr(char *mes){
 
 	pos = mes;
 	lf = strchr(pos, '\n');
-	while ((lf != NULL) && (--lf != "\r")) {
+	while ((lf != NULL) && (*(--lf) != '\r')) {
 		backup=malloc(strlen(lf)+2);
+		if (!backup) {
+			printf("failed to allocate memory\n");
+			exit_code(255);
+		}
 		strcpy(backup, lf+1);
 		//strncpy(lf, "\r", 1);
 		*(lf+1) = '\r';
@@ -209,6 +230,24 @@ void insert_cr(char *mes){
 	}
 }
 
+/* sipmly swappes the content of the two buffers */
+void swap_buffers(char *fst, char *snd) {
+	char *tmp;
+
+	if (fst == snd)
+		return;
+	tmp = malloc(strlen(fst)+1);
+	if (!tmp) {
+		printf("failed to allocate memory\n");
+		exit_code(255);
+	}
+	memset(tmp, 0, strlen(fst)+1);
+	strcpy(tmp, fst);
+	strcpy(fst, snd);
+	strcpy(snd, tmp);
+	free(tmp);
+}
+
 /* trashes one character in buff randomly */
 void trash_random(char *message)
 {
@@ -217,9 +256,9 @@ void trash_random(char *message)
 	char *position;
 
 	t=(float)rand()/RAND_MAX;
-	r=t * (float)strlen(message);
+	r=(int)(t * (float)strlen(message));
 	position=message+r;
-	r=t*(float)255;
+	r=(int)(t*(float)255);
 	*position=(char)r;
 	if (verbose > 2)
 		printf("request:\n%s\n", message);
