@@ -1,7 +1,8 @@
 /*
- * $Id: helper.h,v 1.3 2004/10/08 17:26:00 calrissian Exp $
+ * $Id: helper.h,v 1.6 2005/04/10 20:57:28 calrissian Exp $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
+ * Copyright (C) 2004-2005 Nils Ohlmeier
  *
  * This file belongs to sipsak, a free sip testing tool.
  *
@@ -19,7 +20,11 @@
 #ifndef SIPSAK_HELPER_H
 #define SIPSAK_HELPER_H
 
-#include <sys/time.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#else
+# include <time.h>
+#endif
 
 long getaddress(char *host);
 
@@ -29,6 +34,8 @@ void replace_string(char *mes, char *search, char *replacement);
 
 void insert_cr(char *mes);
 
+void swap_buffers(char *fst, char *snd);
+
 void trash_random(char *message);
 
 double deltaT(struct timeval *t1p, struct timeval *t2p);
