@@ -1,8 +1,8 @@
 /*
- * $Id: shoot.c,v 1.33 2004/09/19 23:49:17 jiri Exp $
+ * $Id: shoot.c,v 1.45 2005/04/11 20:27:59 calrissian Exp $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
- * Copyright (C) 2004 Nils Ohlmeier
+ * Copyright (C) 2004-2005 Nils Ohlmeier
  *
  * This file belongs to sipsak, a free sip testing tool.
  *
@@ -17,32 +17,62 @@
  * GNU General Public License for more details.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-#include <limits.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <regex.h>
-#include <sys/socket.h>
-#include <sys/poll.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include "sipsak.h"
+
+#ifdef HAVE_LIMITS_H
+# include <limits.h>
+#endif
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif /* TIME_WITH_SYS_TIME */
+#ifdef HAVE_UNISTD_H
+# ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+# endif
+# include <unistd.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+# include <sys/socket.h>
+#endif
+#ifdef HAVE_SYS_POLL_H
+# include <sys/poll.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#endif
+#ifdef HAVE_SYS_SELECT_H
+# include <sys/select.h>
+#endif
 
 #include "shoot.h"
 
 #ifdef RAW_SUPPORT
-#include <netinet/in_systm.h>
-#include <netinet/ip.h>
-#include <netinet/ip_icmp.h>
+# ifdef HAVE_NETINET_IN_SYSTM_H 
+#  include <netinet/in_systm.h>
+# endif
+# ifdef HAVE_NETINET_IP_H
+#  include <netinet/ip.h>
+# endif
+# ifdef HAVE_NETINET_IP_ICMP_H
+#  include <netinet/ip_icmp.h>
+# endif
+# ifdef HAVE_NETINET_UDP_H
+#  define __FAVOR_BSD
+#  include <netinet/udp.h>
+# endif
+#endif /* RAW_SUPPORT */
 
-#define __FAVOR_BSD
-#include <netinet/udp.h>
-#endif
-
-#include "sipsak.h"
-#include "request.h"
+#include "request.h"   
 #include "auth.h"
 #include "header_f.h"
 #include "helper.h"
@@ -69,7 +99,7 @@ bouquets and brickbats to farhan@hotfoon.com
 
 inline static void on_success(char *reply)
 {
-	if (re && regexec(re, reply, 0, 0, 0)==REG_NOMATCH) {
+	if ((reply != NULL) && re && regexec(re, reply, 0, 0, 0)==REG_NOMATCH) {
 		fprintf(stderr, "error: RegExp failed\n");
 		exit_code(32);
 	} else {
@@ -87,7 +117,7 @@ void shoot(char *buff)
 	struct pollfd 	sockerr;
 	int redirected, retryAfter, nretries;
 	int usock, csock, i, len, ret;
-	int dontsend, cseqtmp, rand_tmp;
+	int dontsend, dontrecv, cseqtmp, rand_tmp;
 	int rem_rand, retrans_r_c, retrans_s_c;
 	int randretrys = 0;
 	int cseqcmp = 0;
@@ -120,7 +150,7 @@ void shoot(char *buff)
 
 	/* initalize some local vars */
 	redirected = 1;
-	dontsend=retrans_r_c=retrans_s_c = 0;
+	dontsend=dontrecv=retrans_r_c=retrans_s_c = 0;
 	big_delay=tmp_delay = 0;
 	delaytime.tv_sec = 0;
 	delaytime.tv_usec = 0;
@@ -153,7 +183,7 @@ void shoot(char *buff)
 	/* try to create the raw socket */
 	rawsock = (int)socket(PF_INET, SOCK_RAW, IPPROTO_ICMP);
 	if (rawsock==-1) {
-		if (verbose)
+		if (verbose>0)
 			printf("Warning: need raw socket (root privileges) to receive all ICMP errors\n");
 #endif
 		/* create the connected socket as a primitve alternative to the 
@@ -190,7 +220,7 @@ void shoot(char *buff)
 		}
 	}
 
-	if (replace_b){
+	if (replace_b == 1){
 		replace_string(buff, "$dsthost$", domainname);
 		replace_string(buff, "$srchost$", fqdn);
 		lport_str=malloc(6);
@@ -217,19 +247,19 @@ void shoot(char *buff)
 	regcomp(&tmhexp, "^SIP/[0-9]\\.[0-9] 483 ", 
 		REG_EXTENDED|REG_NOSUB|REG_ICASE); 
 
-	if (usrloc||invite||message){
+	if (usrloc == 1||invite == 1||message == 1){
 		/* calculate the number of required steps and create initial mes */
-		if (usrloc) {
-			if (invite)
+		if (usrloc == 1) {
+			if (invite == 1)
 				nretries=4*(nameend-namebeg)+4;
-			else if (message)
+			else if (message == 1)
 				nretries=3*(nameend-namebeg)+3;
 			else
 				nretries=2*(nameend-namebeg)+2;
 			create_msg(buff, REQ_REG);
 			usrlocstep=REG_REP;
 		}
-		else if (invite) {
+		else if (invite == 1) {
 			nretries=3*(nameend-namebeg)+3;
 			create_msg(buff, REQ_INV);
 			usrlocstep=INV_RECV;
@@ -244,7 +274,7 @@ void shoot(char *buff)
 		}
 		cseqcmp=1;
 	}
-	else if (trace){
+	else if (trace == 1){
 		/* for trace we need some spezial initis */
 		if (maxforw!=-1)
 			nretries=maxforw;
@@ -255,19 +285,19 @@ void shoot(char *buff)
 		create_msg(buff, REQ_OPT);
 		add_via(buff);
 	}
-	else if (flood){
+	else if (flood == 1){
 		/* this should be the max of an (32 bit) int without the sign */
 		if (namebeg==-1) namebeg=INT_MAX;
 		nretries=namebeg;
 		namebeg=1;
 		create_msg(buff, REQ_FLOOD);
 	}
-	else if (randtrash){
+	else if (randtrash == 1){
 		randretrys=0;
 		namebeg=1;
 		create_msg(buff, REQ_RAND);
-		nameend=strlen(buff);
-		if (trashchar){
+		nameend=(int)strlen(buff);
+		if (trashchar == 1){
 			if (trashchar < nameend)
 				nameend=trashchar;
 			else
@@ -279,20 +309,20 @@ void shoot(char *buff)
 	}
 	else {
 		/* for non of the modes we also need some inits */
-		if (!file_b) {
+		if (file_b == 0) {
 			namebeg=1;
 			create_msg(buff, REQ_OPT);
 		}
 		/* retryAfter = retryAfter / 10; */
 		if(maxforw!=-1)
 			set_maxforw(buff);
-		if(via_ins)
+		if(via_ins == 1)
 			add_via(buff);
 	}
 
 	/* if we got a redirect this loop ensures sending to the 
 	   redirected server*/
-	while (redirected) {
+	while (redirected == 1) {
 		/* we don't want to send for ever */
 		redirected=0;
 
@@ -315,11 +345,11 @@ void shoot(char *buff)
 		   mode */
 		for (i = 0; i <= nretries; i++)
 		{
-			if (trace) {
+			if (trace == 1) {
 				set_maxforw(buff);
 			}
 			/* some initial output */
-			else if ((usrloc||invite||message) && (verbose > 1) && !dontsend) {
+			else if ((usrloc == 1||invite == 1||message == 1) && (verbose > 1) && (dontsend == 0)) {
 				switch (usrlocstep) {
 					case REG_REP:
 						if (nameend>0)
@@ -332,7 +362,7 @@ void shoot(char *buff)
 						if (nameend>0)
 							printf("inviting user %s%i... ", username, namebeg);
 						else
-							printf("invitng user %s... ", username);
+							printf("inviting user %s... ", username);
 						break;
 					case INV_OK_RECV:
 						printf("sending invite reply... ");
@@ -362,16 +392,15 @@ void shoot(char *buff)
 						break;
 				}
 			}
-			else if (flood && verbose) {
+			else if (flood == 1 && verbose > 0) {
 				printf("flooding message number %i\n", i+1);
 			}
-			else if (randtrash && verbose) {
+			else if (randtrash == 1 && verbose > 0) {
 				printf("message with %i randomized chars\n", i+1);
 				if (verbose > 2)
 					printf("request:\n%s\n", buff);
 			}
-			else if (!trace && !usrloc && !flood && !randtrash && (verbose > 1)
-						&& !dontsend){
+			else if (trace == 0 && usrloc == 0 && flood == 0 && randtrash == 0 && (verbose > 1)	&& dontsend == 0){
 				printf("** request **\n%s\n", buff);
 			}
 
@@ -381,7 +410,7 @@ void shoot(char *buff)
 				sleep_ms_s.tv_nsec = (rand_tmp % 1000) * 1000;
 			}
 
-			if (! dontsend) {
+			if (dontsend == 0) {
 				/* lets fire the request to the server and store when we did */
 				if (csock == -1) {
 					ret = sendto(usock, buff, strlen(buff), 0, (struct sockaddr *)&addr, sizeof(addr));
@@ -402,22 +431,39 @@ void shoot(char *buff)
 			}
 
 			/* in flood we are only interested in sending so skip the rest */
-			if (!flood) {
-				/* set the timeout and wait for a response */
-				tv.tv_sec = retryAfter/1000;
-				tv.tv_usec = (retryAfter % 1000) * 1000;
+			if (flood == 0) {
+				if (! dontrecv) {
+					/* set the timeout and wait for a response */
+					tv.tv_sec = retryAfter/1000;
+					tv.tv_usec = (retryAfter % 1000) * 1000;
 
-				FD_ZERO(&fd);
-				FD_SET(usock, &fd); 
-				if (csock != -1)
-					FD_SET(csock, &fd); 
+					FD_ZERO(&fd);
+					FD_SET(usock, &fd); 
+					if (csock != -1)
+						FD_SET(csock, &fd); 
 #ifdef RAW_SUPPORT
-				if (rawsock != -1)
-					FD_SET(rawsock, &fd); 
+					if (rawsock != -1)
+						FD_SET(rawsock, &fd); 
 #endif
 
-				ret = select(FD_SETSIZE, &fd, NULL, NULL, &tv);
-				(void)gettimeofday(&recvtime, &tz);
+					ret = select(FD_SETSIZE, &fd, NULL, NULL, &tv);
+					(void)gettimeofday(&recvtime, &tz);
+				}
+				else {
+					i--;
+					dontrecv = 0;
+					if (strncmp(buff, "ACK", 3) == 0) {
+						swap_buffers(buff, ack);
+						increase_cseq(buff);
+					}
+					if (usrlocstep == INV_OK_RECV) {
+						increase_cseq(confirm);
+						increase_cseq(ack);
+						usrlocstep = INV_RECV;
+					}
+					continue;
+				}
+
 				if (ret == 0)
 				{
 					/* store the time of our first send */
@@ -439,25 +485,25 @@ void shoot(char *buff)
 								recv(csock, reply, strlen(reply), 0);
 							printf("\n");
 							perror("send failure");
-							if (randtrash) 
+							if (randtrash == 1) 
 								printf ("last message before send failure:"
 									"\n%s\n", buff);
 							exit_code(3);
 						}
 					}
 					/* printout that we did not received anything */
-					if (trace) {
+					if (trace == 1) {
 						printf("%i: timeout after %i ms\n", i, 
 									retryAfter);
 						i--;
 					}
-					else if (usrloc||invite||message) {
+					else if (usrloc == 1||invite == 1||message == 1) {
 						printf("timeout after %i ms\n", retryAfter);
 						i--;
 					}
-					else if (verbose) printf("** timeout after %i ms**\n", 
+					else if (verbose>0) printf("** timeout after %i ms**\n", 
 										retryAfter);
-					if (randtrash) {
+					if (randtrash == 1) {
 						printf("did not get a response on this request:"
 							"\n%s\n", buff);
 						if (i+1 < nameend) {
@@ -476,10 +522,12 @@ void shoot(char *buff)
 						}
 					}
 					senddiff = deltaT(&starttime, &recvtime);
-					if (senddiff > 64 * SIP_T1) {
-						if (verbose)
+					if (senddiff > (float)64 * (float)SIP_T1) {
+						if (verbose>0)
 							printf("*** giving up, no response after %.3f ms\n",
 								senddiff);
+						else if (timing)
+							printf("%.3f ms\n", senddiff);
 						exit_code(3);
 					}
 					/* set retry time according to RFC3261 */
@@ -499,8 +547,7 @@ void shoot(char *buff)
 				}
 				else if (FD_ISSET(usock, &fd) || ((csock != -1) && FD_ISSET(csock, &fd))) {
 					/* no timeout, no error ... something has happened :-) */
-				 	if (!trace && !usrloc && !invite && !message && !randtrash 
-						&& (verbose > 1))
+				 	if (trace == 0 && usrloc ==0 && invite == 0 && message == 0 && randtrash == 0 && (verbose > 1))
 						printf ("\nmessage received:\n");
 				}
 #ifdef RAW_SUPPORT
@@ -576,7 +623,7 @@ void shoot(char *buff)
 #endif
 				if(ret > 0)
 				{
-					reply[ret] = 0;
+					reply[ret] = '\0';
 					/* store the time of our first send */
 					if (i==0)
 						memcpy(&firstsendt, &sendtime, sizeof(struct timeval));
@@ -589,20 +636,20 @@ void shoot(char *buff)
 						delaytime.tv_usec = 0;
 					}
 					/* check for old CSeq => ignore retransmission */
-					if (!usrloc && !invite && !message)
+					if (usrloc == 0 && invite == 0 && message == 0)
 						cseqcmp = namebeg;
 					cseqtmp = cseq(reply);
 					if ((0 < cseqtmp) && (cseqtmp < cseqcmp)) {
-						if (verbose)
+						if (verbose>0)
 							printf("ignoring retransmission\n");
 						retrans_r_c++;
 						dontsend = 1;
 						continue;
 					}
 					/* lets see if received a redirect */
-					if (redirects && regexec(&redexp, reply, 0, 0, 0)==0) {
+					if (redirects == 1 && regexec(&redexp, reply, 0, 0, 0)==0) {
 						printf("** received redirect ");
-						if (warning_ext) {
+						if (warning_ext == 1) {
 							printf("from ");
 							warning_extract(reply);
 							printf("\n");
@@ -614,8 +661,8 @@ void shoot(char *buff)
 							REG_EXTENDED|REG_NOSUB|REG_ICASE);
 						if (regexec(&redexp, reply, 0, 0, 0)==0) {
 							/* try to find the contact in the redirect */
-							if ((foo=strstr(reply, "Contact"))==NULL &&
-								(foo=strstr(reply, "\nm:"))==NULL ) {
+							if ((foo=STRSTR(reply, CONT_STR))==NULL &&
+								(foo=STRSTR(reply, "\nCONT_SHORT_STR"))==NULL ) {
 								printf("error: cannot find Contact in this "
 									"redirect:\n%s\n", reply);
 								exit_code(3);
@@ -624,10 +671,10 @@ void shoot(char *buff)
 							if ((contact=strchr(foo, '\r'))!=NULL 
 							&& contact<crlf)
 								crlf=contact;
-							bar=malloc(crlf-foo+1);
-							strncpy(bar, foo, crlf-foo);
+							bar=malloc((size_t)(crlf-foo+1));
+							strncpy(bar, foo, (size_t)(crlf-foo));
 							*(bar+(crlf-foo))='\0';
-							if ((contact=strstr(bar, "sip"))==NULL) {
+							if ((contact=STRSTR(bar, "sip"))==NULL) {
 								printf("error: cannot find sip in the Contact "
 									"of this redirect:\n%s\n", reply);
 								exit_code(3);
@@ -643,7 +690,7 @@ void shoot(char *buff)
 									*foo='\0';
 									foo++;
 									rport = atoi(foo);
-									if (!rport) {
+									if (rport == 0) {
 										printf("error: cannot handle the port "
 											"in the uri in Contact:\n%s\n", 
 											reply);
@@ -658,7 +705,7 @@ void shoot(char *buff)
 								}
 								/* get the new destination IP*/
 								address = getaddress(crlf);
-								if (!address){
+								if (address == 1){
 									printf("error: cannot determine host "
 										"address from Contact of redirect:"
 										"\n%s\n", reply);
@@ -683,8 +730,8 @@ void shoot(char *buff)
 					} /* if redircts... */
 					else if (regexec(&authexp, reply, 0, 0, 0)==0) {
 						if (!username) {
-							printf("error: received 401 but cannot "
-								"authentication without a username\n");
+							printf("%s\nerror: received 401 but cannot "
+								"authentication without a username\n", reply);
 							exit_code(2);
 						}
 						/* prevents a strange error */
@@ -693,9 +740,16 @@ void shoot(char *buff)
 						insert_auth(buff, reply);
 						if (verbose > 2)
 							printf("\nreceived:\n%s\n", reply);
-						increase_cseq(buff);
+						if (strncmp(buff, "INVITE", 6) == 0) {
+							build_ack(buff, reply, ack);
+							swap_buffers(buff, ack);
+							dontrecv = 1;
+						}
+						else {
+							increase_cseq(buff);
+						}
 					} /* if auth...*/
-					else if (trace) {
+					else if (trace == 1) {
 						if (regexec(&tmhexp, reply, 0, 0, 0)==0) {
 							/* we received 483 to many hops */
 							printf("%i: ", i);
@@ -706,6 +760,10 @@ void shoot(char *buff)
 							else {
 								warning_extract(reply);
 								crlf=strchr(reply, '\n');
+								if (!crlf) {
+									printf("failed to find newline\n");
+									exit_code(254);
+								}
 								*crlf='\0';
 								printf("(%.3f ms) %s\n", 
 									deltaT(&sendtime, &recvtime), reply);
@@ -727,6 +785,10 @@ void shoot(char *buff)
 							else {
 								warning_extract(reply);
 								crlf=strchr(reply, '\n');
+								if (!crlf) {
+									printf("failed to find newline\n");
+									exit_code(254);
+								}
 								*crlf='\0';
 								printf("(%.3f ms) %s\n", 
 									deltaT(&sendtime, &recvtime), reply);
@@ -742,13 +804,17 @@ void shoot(char *buff)
 							else printf("\t");
 							warning_extract(reply);
 							crlf=strchr(reply,'\n');
+							if (!crlf) {
+								printf("failed to find newline\n");
+								exit_code(254);
+							}
 							*crlf='\0';
 							crlf++;
-							contact=strstr(crlf, "Contact");
-							if (!contact)
-								contact=strstr(crlf, "\nm:");
 							printf("(%.3f ms) %s\n", 
 								deltaT(&sendtime, &recvtime), reply);
+							contact=STRSTR(crlf, CONT_STR);
+							if (!contact)
+								contact=STRSTR(crlf, "\nCONT_SHORT_STR");
 							if (contact){
 								crlf=strchr(contact,'\n');
 								*crlf='\0';
@@ -763,7 +829,7 @@ void shoot(char *buff)
 								exit_code(1);
 						}
 					} /* if trace ... */
-					else if (usrloc||invite||message) {
+					else if (usrloc == 1||invite == 1||message == 1) {
 						if (regexec(&proexp, reply, 0, 0, 0)==0) {
 							if (verbose > 2)
 								printf("\nignoring provisinal "
@@ -788,23 +854,23 @@ void shoot(char *buff)
 										"above). aborting\n", reply);
 									exit_code(1);
 								}
-								if (!invite && !message) {
+								if (invite == 0 && message == 0) {
 									if (namebeg==nameend) {
-										if (verbose) 
+										if (verbose>0) 
 											printf("\nAll usrloc tests"
 											" completed successful.\nreceived"
 											" last message %.3f ms after first"
 											" request (test duration).\n", 
 											deltaT(&firstsendt, &recvtime));
-										if (big_delay && verbose)
+										if (big_delay>0 && verbose>0)
 											printf("biggest delay between "
 												"request and response was %.3f"
 												" ms\n", big_delay);
-										if (retrans_r_c && verbose)
+										if (retrans_r_c>0 && verbose>0)
 											printf("%i retransmission(s) "
 												"received from server.\n", 
 												retrans_r_c);
-										if (retrans_s_c && verbose) {
+										if (retrans_s_c>0 && verbose>0) {
 											printf("%i time(s) the timeout of "
 												"%i ms exceeded and request was"
 												" retransmitted.\n", 
@@ -812,12 +878,14 @@ void shoot(char *buff)
 											if (retrans_s_c > nagios_warn)
 												exit_code(4);
 										}
+										if (timing) printf("%.3f ms\n",
+															deltaT(&firstsendt, &recvtime));
 										on_success(reply);
 									}
 									/* lets see if we deceid to remove a 
 									   binding (case 6)*/
 									rem_rand=rand();
-									if (!rand_rem ||
+									if (rand_rem == 0||
 										((float)rem_rand/RAND_MAX) 
 											> USRLOC_REMOVE_PERCENT) {
 										namebeg++;
@@ -837,12 +905,12 @@ void shoot(char *buff)
 										usrlocstep=UNREG_REP;
 									}
 								}
-								else if (invite) {
+								else if (invite == 1) {
 									create_msg(buff, REQ_INV);
 									cseqcmp++;
 									usrlocstep=INV_RECV;
 								}
-								else if (message) {
+								else if (message == 1) {
 									create_msg(buff, REQ_MES);
 									cseqcmp++;
 									usrlocstep=MES_RECV;
@@ -874,7 +942,7 @@ void shoot(char *buff)
 							case INV_OK_RECV:
 								/* did we received our ok ? */
 								if (strncmp(reply, INV_STR, INV_STR_LEN)==0) {
-									if (verbose)
+									if (verbose>0)
 										printf("ignoring INVITE "
 											"retransmission\n");
 									retrans_r_c++;
@@ -912,28 +980,28 @@ void shoot(char *buff)
 										printf("\t\tack received\n");
 									if (verbose > 2)
 										printf("\n%s\n", reply);
-									if (verbose && nameend>0)
+									if (verbose>0 && nameend>0)
 										printf("usrloc for %s%i completed "
 											"successful\n", username, namebeg);
-									else if (verbose)
+									else if (verbose>0)
 										printf("usrloc for %s completed "
 											"successful\n", username);
 									if (namebeg==nameend) {
-										if (verbose)
+										if (verbose>0)
 											printf("\nAll usrloc tests completed "
 												"successful.\nreceived last message"
 												" %.3f ms after first request (test"
 												" duration).\n", deltaT(&firstsendt,
 												 &recvtime));
-										if (big_delay)
+										if (big_delay>0)
 											printf("biggest delay between "
 												"request and response was %.3f"
 												" ms\n", big_delay);
-										if (retrans_r_c)
+										if (retrans_r_c>0)
 											printf("%i retransmission(s) "
 												"received from server.\n", 
 												retrans_r_c);
-										if (retrans_s_c) {
+										if (retrans_s_c>0) {
 											printf("%i time(s) the timeout of "
 												"%i ms exceeded and request was"
 												" retransmitted.\n", 
@@ -943,11 +1011,11 @@ void shoot(char *buff)
 										}
 										on_success(reply);
 									}
-									if (usrloc) {
+									if (usrloc == 1) {
 										/* lets see if we deceid to remove a 
 										   binding (case 6)*/
 										rem_rand=rand();
-										if (!rand_rem ||
+										if (rand_rem == 0||
 											((float)rem_rand/RAND_MAX) 
 												> USRLOC_REMOVE_PERCENT) {
 											namebeg++;
@@ -991,7 +1059,7 @@ void shoot(char *buff)
 								if (!strncmp(reply, messusern, 
 									strlen(messusern))) {
 									if (verbose > 1) {
-										crlf=strstr(reply, "\r\n\r\n");
+										crlf=STRSTR(reply, "\r\n\r\n");
 										crlf=crlf+4;
 										printf("  received message\n  "
 											"'%s'\n", crlf);
@@ -1014,7 +1082,7 @@ void shoot(char *buff)
 								/* we sent our reply on the message and
 								   look if this is also forwarded to us */
 								if (strncmp(reply, MES_STR, MES_STR_LEN)==0) {
-									if (verbose)
+									if (verbose>0)
 										printf("ignoring MESSAGE "
 											"retransmission\n");
 									retrans_r_c++;
@@ -1024,28 +1092,28 @@ void shoot(char *buff)
 								if (regexec(&okexp, reply, 0, 0, 0)==0) {
 									if (verbose > 1)
 										printf("  reply received\n\n");
-									else if (verbose && nameend>0)
+									else if (verbose>0 && nameend>0)
 										printf("usrloc for %s%i completed "
 											"successful\n", username, namebeg);
-									else if (verbose)
+									else if (verbose>0)
 										printf("usrloc for %s completed "
 											"successful\n", username);
 									if (namebeg==nameend) {
-										if (verbose)
+										if (verbose>0)
 											printf("\nAll usrloc tests completed "
 												"successful.\nreceived last message"
 												" %.3f ms after first request (test"
 												" duration).\n", deltaT(&firstsendt,
 												 &recvtime));
-										if (big_delay)
+										if (big_delay>0)
 											printf("biggest delay between "
 												"request and response was %.3f"
 												" ms\n", big_delay);
-										if (retrans_r_c)
+										if (retrans_r_c>0)
 											printf("%i retransmission(s) "
 												"received from server.\n", 
 												retrans_r_c);
-										if (retrans_s_c) {
+										if (retrans_s_c>0) {
 											printf("%i time(s) the timeout of "
 												"%i ms exceeded and request was"
 												" retransmitted.\n", 
@@ -1055,11 +1123,11 @@ void shoot(char *buff)
 										}
 										on_success(reply);
 									}
-									if (usrloc) {
+									if (usrloc == 1) {
 										/* lets see if we deceid to remove a 
 										   binding (case 6)*/
 										rem_rand=rand();
-										if (!rand_rem ||
+										if (rand_rem>0 ||
 											((float)rem_rand/RAND_MAX) 
 												> USRLOC_REMOVE_PERCENT) {
 											namebeg++;
@@ -1088,7 +1156,7 @@ void shoot(char *buff)
 									}
 								}
 								else {
-									if (verbose) {
+									if (verbose>0) {
 										if (mes_body)
 											printf("\nreceived:\n%s\nerror: did"
 												" not received 200 for the "
@@ -1108,7 +1176,7 @@ void shoot(char *buff)
 								break;
 							case UNREG_REP:
 								if (strncmp(reply, MES_STR, MES_STR_LEN)==0) {
-									if (verbose)
+									if (verbose>0)
 										printf("ignoring MESSAGE "
 											"retransmission\n");
 									retrans_r_c++;
@@ -1117,10 +1185,10 @@ void shoot(char *buff)
 								}
 								if (regexec(&okexp, reply, 0, 0, 0)==0) {
 									if (verbose > 1) printf("   OK\n\n");
-									else if (verbose && nameend>0)
+									else if (verbose>0 && nameend>0)
 										printf("Binding removal for %s%i "
 											"successful\n", username, namebeg);
-									else if (verbose)
+									else if (verbose>0)
 										printf("Binding removal for %s "
 											"successful\n", username);
 									namebeg = rem_namebeg;
@@ -1148,7 +1216,7 @@ void shoot(char *buff)
 						}
 						}
 					}
-					else if (randtrash) {
+					else if (randtrash == 1) {
 						/* in randomzing trash we are expexting 4?? error codes
 						   everything else should not be normal */
 						if (regexec(&errexp, reply, 0, 0, 0)==0) {
@@ -1156,7 +1224,7 @@ void shoot(char *buff)
 								printf("received:\n%s\n", reply);
 							if (verbose > 1) {
 								printf("received expected 4xx ");
-								if (warning_ext) {
+								if (warning_ext == 1) {
 									printf ("from ");
 									warning_extract(reply);
 									printf("\n");
@@ -1201,6 +1269,10 @@ void shoot(char *buff)
 										deltaT(&firstsendt, &recvtime), 
 										deltaT(&sendtime, &recvtime));
 								crlf=strchr(reply, '\n');
+								if (!crlf) {
+									printf("failed to find newline\n");
+									exit_code(254);
+								}
 								*crlf='\0';
 								printf("   %s\n   provisional received; still"
 									" waiting for a final response\n", reply);
@@ -1221,10 +1293,16 @@ void shoot(char *buff)
 										deltaT(&firstsendt, &recvtime), 
 										deltaT(&sendtime, &recvtime));
 								crlf=strchr(reply, '\n');
+								if (!crlf) {
+									printf("failed to find newline\n");
+									exit_code(254);
+								}
 								*crlf='\0';
 								printf("   %s\n   final received\n", reply);
 							}
-							else if (verbose) printf("%s\n", reply);
+							else if (verbose>0) printf("%s\n", reply);
+							else if (timing) printf("%.3f ms\n", 
+										deltaT(&firstsendt, &recvtime));
 							if (regexec(&okexp, reply, 0, 0, 0)==0)
 								on_success(reply);
 							else
@@ -1234,7 +1312,7 @@ void shoot(char *buff)
 		
 				} /* ret > 0 */
 				else {
-					if (usrloc)
+					if (usrloc == 1)
 						printf("failed\n");
 					perror("socket error");
 					exit_code(3);
@@ -1257,11 +1335,11 @@ void shoot(char *buff)
 		} /* for nretries */
 
 	} /* while redirected */
-	if (randtrash) exit_code(0);
+	if (randtrash == 1) exit_code(0);
 	printf("** give up retransmissioning....\n");
-	if (retrans_r_c && (verbose > 1))
+	if (retrans_r_c>0 && (verbose > 1))
 		printf("%i retransmissions received during test\n", retrans_r_c);
-	if (retrans_s_c && (verbose > 1))
+	if (retrans_s_c>0 && (verbose > 1))
 		printf("sent %i retransmissions during test\n", retrans_s_c);
 	exit_code(3);
 }
