@@ -1,8 +1,8 @@
 /*
- * $Id: sipsak.c,v 1.76 2004/10/08 18:07:14 calrissian Exp $
+ * $Id: sipsak.c,v 1.88 2005/04/10 20:57:28 calrissian Exp $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
- * Copyright (C) 2004 Nils Ohlmeier
+ * Copyright (C) 2004-2005 Nils Ohlmeier
  *
  * This file belongs to sipsak, a free sip testing tool.
  *
@@ -30,29 +30,35 @@
    dropped
 */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
-#include <regex.h>
-#include <sys/wait.h>
-
-#include "config.h"
+#include "sipsak.h"
 
+#ifdef HAVE_UNISTD_H
+# ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+# endif
+# include <unistd.h>
+#endif
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+#ifdef HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
 #ifdef HAVE_GETOPT_H
-#include <getopt.h>
+# include <getopt.h>
 #endif
 
-#include "sipsak.h"
 #include "helper.h"
 #include "shoot.h"
 #include "exit_code.h"
 
-/* regular expression for matching received replies */
-regex_t* re;
-
-
 static void sigchld_handler(int signo)
 {
 	int chld_status;
@@ -65,7 +71,7 @@ static void sigchld_handler(int signo)
 void print_version() {
 	printf("%s %s by Nils Ohlmeier\n", PACKAGE_NAME, PACKAGE_VERSION);
 	printf(" Copyright (C) 2002-2004 FhG Fokus\n");
-	printf(" Copyright (C) 2004 Nils Ohlmeier\n");
+	printf(" Copyright (C) 2004-2005 Nils Ohlmeier\n");
 	printf(" report bugs to %s\n\n", PACKAGE_BUGREPORT);
 	printf(
 		" shoot  : sipsak [-f FILE] [-L] -s SIPURI\n"
@@ -118,7 +124,7 @@ void print_long_help() {
 		"  --outbound-proxy=HOSTNAME  request target (outbound proxy)\n"
 		"  --hostname=HOSTNAME        overwrites the local hostname in all headers\n"
 		"  --max-forwards=NUMBER      the value for the max-forwards header field\n"
-		"  --numeric                  use IPs instead of FQDN in the Via-Line\n");
+		"  --numeric                  use FQDN instead of IPs in the Via-Line\n");
 	printf("  --processes=NUMBER         Divide the workflow among the number of processes\n"
 		"  --auth-username=STRING     username for authentication\n"
 		);
@@ -137,8 +143,9 @@ void print_long_help() {
 		"  --disposition=STRING       Content-Disposition value\n"
 		"  --search=REGEXP            search for a RegExp in replies and return error\n"
 		"                             on failfure\n"
+		"  --timing                   print the timing informations at the end\n"
 		);
-	exit(0);
+	exit_code(0);
 }
 
 /* prints out some usage help and exits */
@@ -178,7 +185,7 @@ void print_help() {
 	printf(
 		"  -H HOSTNAME       overwrites the local hostname in all headers\n"
 		"  -m NUMBER         the value for the max-forwards header field\n"
-		"  -n                use IPs instead of FQDN in the Via-Line\n"
+		"  -n                use FQDN instead of IPs in the Via-Line\n"
 		"  -i                deactivate the insertion of a Via-Line\n"
 		"  -a PASSWORD       password for authentication\n"
 		"                      (if omitted password=\"\")\n"
@@ -197,16 +204,21 @@ void print_help() {
 		"  -B STRING         send a message with string as body\n"
 		"  -O STRING         Content-Disposition value\n"
 		"  -P NUMBER         Number of processes to start\n"
+		"  -A                print timing informations\n"
 		);
-		exit(0);
+		exit_code(0);
 }
 
 int main(int argc, char *argv[])
 {
 	FILE	*pf;
 	char	buff[BUFSIZE];
-	int		length, c, i, j;
+	int	length, c, i, j;
 	char	*delim, *delim2;
+	pid_t 	pid;
+	struct timespec ts;
+	int 	upp;
+
 #ifdef HAVE_GETOPT_LONG
 	int option_index = 0;
 	static struct option l_opts[] = {
@@ -249,15 +261,16 @@ int main(int argc, char *argv[])
 		{"processes", 1, 0, 'P'},
 		{"auth-username", 1, 0, 'u'},
 		{"no-crlf", 0, 0, 'L'},
+		{"timing", 0, 0, 'A'},
 		{0, 0, 0, 0}
 	};
 #endif
 	/* some initialisation to be shure */
 	file_b=uri_b=trace=lport=usrloc=flood=verbose=randtrash=trashchar = 0;
-	numeric=warning_ext=rand_rem=nonce_count=replace_b=invite=message = 0;
-	sleep_ms=empty_contact=nagios_warn = 0;
+	warning_ext=rand_rem=nonce_count=replace_b=invite=message = 0;
+	sleep_ms=empty_contact=nagios_warn=timing = 0;
 	namebeg=nameend=maxforw= -1;
-	via_ins=redirects=crlf=processes  = 1;
+	numeric=via_ins=redirects=fix_crlf=processes  = 1;
 	username=password=replace_str=hostname=contact_uri=mes_body = NULL;
 	con_dis=auth_username = NULL;
 	re = NULL;
@@ -269,17 +282,14 @@ int main(int argc, char *argv[])
 	memset(ack, 0, BUFSIZE);
 	memset(fqdn, 0, FQDN_SIZE);
 	memset(messusern, 0, FQDN_SIZE);
-	pid_t pid;
-	struct timespec ts;
-	int upp;
 
 	if (argc==1) print_help();
 
 	/* lots of command line switches to handle*/
 #ifdef HAVE_GETOPT_LONG
-	while ((c=getopt_long(argc, argv, "a:B:b:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z", l_opts, &option_index)) != EOF){
+	while ((c=getopt_long(argc, argv, "a:AB:b:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z", l_opts, &option_index)) != EOF){
 #else
-	while ((c=getopt(argc,argv,"a:B:b:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z")) != EOF){
+	while ((c=getopt(argc,argv,"a:AB:b:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z")) != EOF){
 #endif
 		switch(c){
 			case 'a':
@@ -287,6 +297,9 @@ int main(int argc, char *argv[])
 				strncpy(password, optarg, strlen(optarg));
 				*(password+strlen(optarg)) = '\0';
 				break;
+			case 'A':
+				timing=1;
+				break;
 			case 'b':
 				if ((namebeg=atoi(optarg))==-1) {
 					printf("error: non-numerical appendix begin for the "
@@ -303,35 +316,27 @@ int main(int argc, char *argv[])
 				if (!strncmp(optarg, "empty", 5) || !strncmp(optarg, "none", 4)) {
 					empty_contact = 1;
 				}
-				else if (!strncmp(optarg,"sip",3)){
-					if ((delim=strchr(optarg,':'))!=NULL){
-						delim++;
-						if ((delim2=strchr(delim,'@'))==NULL){
-							printf("error: missing '@' in Contact uri\n");
-							exit_code(2);
-						}
-						else{
-							if ((delim2-delim)==0){
-								printf("error: REGISTER Contact requires a"
-									" username\n");
-								exit_code(2);
-							}
-							else{
-								contact_uri=malloc(strlen(optarg)+1);
-								memset(contact_uri, 0, strlen(optarg)+1);
-								strncpy(contact_uri, optarg, strlen(optarg));
-							}
-						}
-					}
-					else{
-						printf("error: missing ':' in REGISTER Contact uri\n");
+				else if (((delim=STRSTR(optarg,"sip:"))!=NULL) ||
+					((delim=STRSTR(optarg,"sips:"))!=NULL)) {
+			 		if (strchr(optarg,'@')<delim) {
+						printf("error: missing '@' in Contact uri\n");
 						exit_code(2);
 					}
+					else {
+						contact_uri=malloc(strlen(optarg)+1);
+						memset(contact_uri, 0, strlen(optarg)+1);
+						strncpy(contact_uri, optarg, strlen(optarg));
+					}
+				}
+				else if ((strlen(optarg)==1) && (!strncmp(optarg, "*", 1))) {
+					contact_uri=malloc(strlen(optarg)+1);
+					memset(contact_uri, 0, strlen(optarg)+1);
+					strncpy(contact_uri, optarg, strlen(optarg));
 				}
 				else{
-					printf("error: REGISTER Contact uri doesn't not begin "
-						"with sip or empty\n");
-					exit_code(2);
+				    printf("error: REGISTER Contact uri doesn't not contain "
+					   "sip:, *, or is not empty\n");
+				    exit_code(2);
 				}
 				break;
 			case 'c':
@@ -354,7 +359,7 @@ int main(int argc, char *argv[])
 				flood=1;
 				break;
 			case 'f':
-				if strncmp(optarg, "-", 1) {
+				if (strncmp(optarg, "-", 1)) {
 					/* file is opened in binary mode so that the cr-lf is 
 					   preserved */
 					pf = fopen(optarg, "rb");
@@ -411,7 +416,7 @@ int main(int argc, char *argv[])
 				}
 				break;
 			case 'L':
-				crlf=0;
+				fix_crlf=0;
 				break;
 			case 'm':
 				maxforw=atoi(optarg);
@@ -424,7 +429,7 @@ int main(int argc, char *argv[])
 				message=1;
 				break;
 			case 'n':
-				numeric = 1;
+				numeric = 0;
 				break;
 			case 'N':
 				exit_mode=EM_NAGIOS;
@@ -557,7 +562,7 @@ int main(int argc, char *argv[])
 				break;
 			case 'V':
 				printf("sipsak %s  by Nils Ohlmeier\n Copyright (C) 2002-2004"
-						" FhG Fokus\n Copyright (C) 2004 Nils Ohlmeier\n", 
+						" FhG Fokus\n Copyright (C) 2004-2005 Nils Ohlmeier\n", 
 						SIPSAK_VERSION);
 				printf(" compiled with DEFAULT_RETRYS=%i, DEFAULT_TIMEOUT=%i, FQDN_SIZE=%i",
 						DEFAULT_RETRYS, DEFAULT_TIMEOUT, FQDN_SIZE);
@@ -595,7 +600,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* replace LF with CRLF if we read from a file */
-	if ((file_b) && (crlf)) {
+	if ((file_b) && (fix_crlf)) {
 		insert_cr(buff);
 	}
 	/* lots of conditions to check */
@@ -671,6 +676,7 @@ int main(int argc, char *argv[])
 			}
 		}
 		if (via_ins) {
+			printf("warning: ignoring -i option when in usrloc mode\n");
 			via_ins=0;
 		}
 		if (nameend==-1)
